<?php

namespace Drupal\commerce_rbspayment\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\commerce_price\Calculator;
use Drupal\commerce_price\Entity\Currency;
use Drupal\commerce_price\Price;
use Drupal\commerce_rbspayment\CommerceRBSPaymentApi;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_order\Entity\Order;
use Drupal\commerce_order\Entity\OrderItem;


class PaymentOffsiteForm extends BasePaymentOffsiteForm
{

    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state)
    {
        $form = parent::buildConfigurationForm($form, $form_state);

        /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
        $payment = $this->entity;
        /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
        $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();

        $data = [];

        $payment_gateway_configuration = $payment_gateway_plugin->getConfiguration();

        $params['userName'] = $payment_gateway_configuration['username'];
        $params['password'] = $payment_gateway_configuration['password'];
        $params['isDoubleStaged'] = $payment_gateway_configuration['double_staged'] == 0 ? false : true;
        $params['isTestMode'] =  $payment_gateway_configuration['mode'] == 'live' ? false : true;
        $params['isLogging'] = $payment_gateway_configuration['logging'] == 0 ? false : true;

        $params['isSendOrder'] = $payment_gateway_configuration['send_order'] == 0 ? false : true;
//        $params['ffdVersion'] = $payment_gateway_configuration['ffd_version'];
        $params['taxSystem'] = $payment_gateway_configuration['tax_system'];

        // collect orderBundle
        if ($params['isSendOrder']) {

            $order = $payment->getOrder();
            $dp_items = $order->getItems();

            $items = array();
            $itemsCnt = 1;

            foreach ($dp_items as $key => $value) {

                $itemPrice = round($value->getUnitPrice()->getNumber(), 2) * 100;

                $tax_type = $this->getTaxType($value, $payment_gateway_configuration['tax_type']);

                $item = array(
                    'positionId' => $itemsCnt++,
                    'itemAmount' => $value->getQuantity() * $itemPrice,
                    'name' => $value->getTitle(),
                    'itemPrice' => $itemPrice,
                    'quantity' => array(
                        'value' => $value->getQuantity(),
                        'measure' => 'шт'
                    ),
                    'tax' => array(
                        'taxType' => $tax_type
                    ),
                    'itemCode' => $value->id()
                );

                // FFD 1.05 added
//                if ($params['ffdVersion'] == 'v105') {

                    $attributes = array();
                    $attributes[] = array(
                        "name" => "paymentMethod",
                        "value" => $payment_gateway_configuration['ffd_paymentMethodType']
                    );
                    $attributes[] = array(
                        "name" => "paymentObject",
                        "value" => $payment_gateway_configuration['ffd_paymentObjectType']
                    );

                    $item['itemAttributes']['attributes'] = $attributes;

//                }

                $items[] = $item;
            }

            $order_bundle = array(
                'orderCreationDate' => time(), //$order->changed,
                'customerDetails' => array(
                    'email' => $order->getEmail()
                ),
                'cartItems' => array('items' => $items)
            );
            $params['orderBundle'] = $order_bundle;

        } //orderBundle

        $rbs = new CommerceRBSPaymentApi($params);

        $amount = $payment->getOrder()->getTotalPrice();
        $payment->save();

        $response = $rbs->registerOrder(
            $payment->id(),
            $this->toMinorUnits($amount),
            $form["#return_url"]
//            $form['#exception_url']
        );

        if (!isset($response['errorCode'])) {
            $redirect_url = $response['formUrl'];
            $payment->setRemoteId($response['orderId']);
            $payment->save();
            return $this->buildRedirectForm($form, $form_state, $redirect_url, $data, self::REDIRECT_GET);

        } else {
            addMessage(t('Error # %code: %message', [ //PLUG-4598
                '%code' => $response['errorCode'],
                '%message' => $response['errorMessage']]), 'error');
//            $redirect_url = $form['#exception_url'];
        }


    }

    /**
     * Converts the given amount to its minor units.
     *
     * For example, 9.99 USD becomes 999.
     *
     * @param \Drupal\commerce_price\Price $amount
     *   The amount.
     *
     * @return int
     *   The amount in minor units, as an integer.
     */
    protected function toMinorUnits(Price $amount)
    {
        /** @var \Drupal\commerce_price\Entity\CurrencyInterface $currency */
        $currency = Currency::load($amount->getCurrencyCode());
        $fraction_digits = $currency->getFractionDigits();
        $number = $amount->getNumber();
        if ($fraction_digits > 0) {
            $number = Calculator::multiply($number, pow(10, $fraction_digits));
        }

        return round($number, 0);
    }

    /**
     * @param string $currency_code
     *
     * @return string
     */
    function getCurrencyNumericCode($currency_code)
    {
        $currency = Currency::load($currency_code);
        return $currency->getNumericCode();
    }


    function getTaxType($value, $default_value)
    {
        $item_rate = -1; // wo NDS

        foreach ($value->getAdjustments() as $k => $v) {
            $item_rate = intval($v->getPercentage() * 100); // $tax_data['percentage']
        }

        if ($item_rate == 20) {
            $tax_type = 6;

        } else if ($item_rate == 18) {
            $tax_type = 3;

        } else if ($item_rate == 10) {
            $tax_type = 2;

        } else if ($item_rate == 0) {
            $tax_type = 1;

        } else {
            $tax_type = $default_value;
        }

        return $tax_type;
    }
}
